
/**
 * The dragDropHandler is used to handle drag and drop operations to move elements around on the screen
 */

dragDropHandler =
{
    mode:           'off', //indicate whether a drag operation is currently being performed
    sourceNode:     null, //stores the dom elmenent thats starts the current drag operations
    moveNode:       null, //stores the temporary node used to track the mouse and indicate the drag operation
    targetNode:     null, //stores the node on which the drag operation would be completed if the mouse released
    targetLocation: null, //stores the location value relative to the targetNode that the current drag location indicates
    currentID:      null, //stores the ID value of the dragDropContainer object for the drag source
    dropMarker:     null, //stores the html element temporarily placed in the dom to indicate the current target location
    targets:        null,  //an associative array mapping container IDs to their dragDropContainer objects.
    DEFAULT_MARKER_LOCATIONS: { //indicates the possible marker locations that will be used if not directly specified
        top:    true,
        bottom: true
    }
};


dragDropHandler.initiate = function()
{
    dragDropHandler.mode = 'off';

    //stores the dragDropContainer objects for each drag drop container available on the page
    dragDropHandler.targets = new Object();

    //set up the gloabl event handlers
    WinManager.attachEvent(document, "onmousemove", dragDropHandler.onmousemove);
    WinManager.attachEvent(document, "onmouseup", dragDropHandler.onmouseup);
}

/*
 * Register a container to be managed by this Drag and Drop Handler.
 *
 * The drag functionality can be customised using the parameters here, and also by adding
 * the following optional attributes to each drag item:
 * dd_locations_source  A comma separated string indicating the locations this element can be placed into
 *                      when it is the source of a drag operation
 *                      The available values are 'top', 'bottom', 'left', 'right', 'around', 'into'
 * dd_locations_target  A comma separated string indicating the locations this element supports as the target
 *                      of a drag operation
 *                      The available values are 'top', 'bottom', 'left', 'right', 'around', 'into'
 * dd_marker_sizer_x    A script fragment to eval to determine the container that should be used to size
 *                      the drop marker in the x direction, instead of this component when used as the target for
 *                      a drag. Alternatively this can contain the value 'OWNER-TABLE' for elements inside a table, to
 *                      indicate that the parent table size should be used instead of the component (eg td)
 *                      Other options are 'INCLUDE-PREVIOUS' and 'INCLUDE-FOLLOWING' which add the size of the
 *                      previous or following component as appropriate to the size of this component when determining
 *                      the marker size.
 * dd_marker_sizer_y    As above but to determnine the y size of the marker
 *
 *
 * @param container The HTML element containing all components that could be the target of the drop
 * @param type A string value containing the type of this container.  You will only be able to drag elements
 *             between containers of the same type.
 * @param methodName The name of the method to call after complete a dnd operation to this container.
 * @param relevantClass The class name that all elements within the container must have to be
 *             considered as draggable components/drop targets.  If not provided, only first level children will be used.
 * @param sourceTarget Indicates whether this container provides drag 'source's, 'target's or 'both' (default).
 * @param dropMarkerOffset (Optional) The offset in pixels to draw the drop marker relative to the target location
 * @param sourceLocations (Optional) An array of properties indicating which drag locations can be used for elements
 *                          being dragged from this container. eg {top:true, bottom:true}
 *                          The full set of possible properties is: 'top', 'bottom', 'left', 'right', 'around', 'into'
 * @param targetLocations (Optional) An array of properties indicating which drag locations can be used for elements
 *                          being dragged into this container. eg {top:true, bottom:true}
 *                          The full set of possible properties is: 'top', 'bottom', 'left', 'right', 'around', 'into'
 * @return The ID given to this container in this dragDropHandler
 */
dragDropHandler.attachContainer = function(container, type, methodName, relevantClass, sourceTarget, dropMarkerOffset,
                                            sourceLocations, targetLocations)
{

    var containerID = getPropertyCount(dragDropHandler.targets);
    while (typeof(dragDropHandler.targets[containerID]) != 'undefined') { ++containerID; }

    if ((typeof(relevantClass) == 'undefined') || (relevantClass == null) || (relevantClass == ''))
    {
        var matches = container.childNodes;
        for (var i = 0; i < matches.length; ++i)
        {
            var child = matches.item(i);
            if(child.nodeType == 1) //element node
            {
                child.setAttribute("dnd_handler", containerID);

                WinManager.attachEvent(child, "onmousedown", dndEventHandler);
                WinManager.attachEvent(child, "onmouseup", dndEventHandler);
                WinManager.attachEvent(child, "onmousemove", dndEventHandler);
            }
        }
    }
    else
    {
        var matches = getElementsByClassName(container, relevantClass);
        for (var i = 0; i < matches.length; ++i)
        {
            var node = matches[i];
            node.setAttribute("dnd_handler", containerID);

            WinManager.attachEvent(node, "onmousedown", dndEventHandler);
            WinManager.attachEvent(node, "onmouseup", dndEventHandler);
            WinManager.attachEvent(node, "onmousemove", dndEventHandler);
        }
    }



    container.setAttribute("dnd_handler", containerID);
    container.setAttribute("dnd_type", "container");
    WinManager.attachEvent(container, "onmouseup", dndEventHandler);
    WinManager.attachEvent(container, "onmousemove", dndEventHandler);

    dragDropHandler.targets[containerID] = new dragDropContainer(container, type, methodName, relevantClass,
                                                                 sourceTarget, dropMarkerOffset,
                                                                 sourceLocations, targetLocations);

    return containerID;
}

/**
 * Remove this container from being managed by this drag and drop handler
 * This also removes any event triggers and attributes added during attachement.
 * @param if The ID assigned to the container to remove by this dnd handler
 */
dragDropHandler.detachContainer = function(id)
{
    ddc = dragDropHandler.targets[id]

    if ((typeof(ddc.relevantClass) == 'undefined') || (ddc.relevantClass == null))
    {
        for (var i = 0; i < ddc.container.childNodes.length; ++i)
        {
            var child = ddc.container.childNodes.item(i);
            if(child.nodeType == 1) //element node
            {
                child.removeAttribute("dnd_handler");

                WinManager.detachEvent(child, "onmousedown", dndEventHandler);
                WinManager.detachEvent(child, "onmouseup", dndEventHandler);
                WinManager.detachEvent(child, "onmousemove", dndEventHandler);

            }
        }
    }
    else
    {
        var matches = getElementsByClassName(ddc.container, ddc.relevantClass);
        for (var i = 0; i < matches.length; ++i)
        {
            var node = matches[i];
            node.removeAttribute("dnd_handler", id);

            WinManager.detachEvent(node, "onmousedown", dndEventHandler);
            WinManager.detachEvent(node, "onmouseup", dndEventHandler);
            WinManager.detachEvent(node, "onmousemove", dndEventHandler);
        }
    }

    ddc.container.removeAttribute('dnd_handler');
    ddc.container.removeAttribute('dnd_type');
    WinManager.detachEvent(ddc.container, "onmouseup", dndEventHandler);
    WinManager.detachEvent(ddc.container, "onmousemove", dndEventHandler);

    delete dragDropHandler.targets[id];
}


/**
 * Stops the current event from bubbling up the heirarchy for drag operations.
 * This should be added (as onmousemove) to containers that have both child and parent
 * drag and drop items, and you dont want the parent items to be detected when determining
 * target move locations, and moving between the child items.
 */
dragDropHandler.stopEventBubblingForDrag = function(evt)
{
    if (dragDropHandler.mode == 'moving')
        dragDropHandler.mode = 'processed';
}



dragDropHandler.removeMoveNode = function()
{
    if (dragDropHandler.moveNode != null)
    {
        if (dragDropHandler.moveNode.parentNode != null)
            dragDropHandler.moveNode.parentNode.removeChild(dragDropHandler.moveNode);
        delete dragDropHandler.moveNode;
    }
    dragDropHandler.moveNode = null;
}

dragDropHandler.createMoveNode = function()
{
    //first remove any existing move node
    if (dragDropHandler.moveNode != null)
        dragDropHandler.removeMoveNode();

    if (dragDropHandler.sourceNode != null)
    {
        //clone the source node to create the move node
        dragDropHandler.moveNode = dragDropHandler.sourceNode.cloneNode(true);

        //special handling for certain types of node, as in IE7
        //you cant use absolute positioning on a tr for example
        if (dragDropHandler.moveNode.tagName.toLowerCase() == 'tr')
        {
            var table = document.createElement('table');
            table.appendChild(dragDropHandler.moveNode);
            dragDropHandler.moveNode = table;
        }
        if (dragDropHandler.moveNode.tagName.toLowerCase() == 'table')
        {
            var container = document.createElement('div');
            container.innerHTML = '<table>' +dragDropHandler.moveNode.innerHTML+ '</table>';
            dragDropHandler.moveNode = container;
        }
    }
}

dragDropHandler.removeDropMarker = function()
{
    if (dragDropHandler.dropMarker != null)
    {
        if ((typeof(dragDropHandler.dropMarker.parentNode)) != 'undefined' && (dragDropHandler.dropMarker.parentNode != null))
        {
            dragDropHandler.dropMarker.parentNode.removeChild(dragDropHandler.dropMarker);
        }
        else
        {
            for (prop in dragDropHandler.dropMarker)
            {
                if ((typeof(dragDropHandler.dropMarker[prop].parentNode)) != 'undefined' && (dragDropHandler.dropMarker[prop].parentNode != null))
                {
                    dragDropHandler.dropMarker[prop].parentNode.removeChild(dragDropHandler.dropMarker[prop]);
                }
            }
        }

        delete dragDropHandler.dropMarker;
    }
    dragDropHandler.dropMarker = null;
    dragDropHandler.targetNode = null;
    dragDropHandler.targetLocation == null;
}

/**
 * Creates an HTML component that can be used to show a drop marker for the given location type.
 * The created html element is assigned to dragDropHandler.dropMarker
 */
dragDropHandler.createDropMarker = function(dropMarkerOffset)
{
    var elemCoords = getComponentPosition(dragDropHandler.targetNode, true);
    var markerSizer = dragDropHandler.targetNode;
    var markerSizerCoords = getComponentPosition(markerSizer, true);


    if (dragDropHandler.targetNode.getAttribute('dd_marker_sizer_x') != null)
    {
        var msString = dragDropHandler.targetNode.getAttribute('dd_marker_sizer_x');
        var tempSizer = null;
        if (msString == 'OWNER-TABLE')
        {
            var temp = dragDropHandler.targetNode;
            while (temp != null)
            {
                if (temp.tagName.toLowerCase() == 'table')
                {
                    tempSizer = temp;
                    break;
                }
                temp = temp.parentNode;
            }
        }
        else if (msString == 'INCLUDE-PREVIOUS')
        {
            if (markerSizer.previousSibling)
            {
                var prevCoords = getComponentPosition(markerSizer.previousSibling, true);
                markerSizerCoords.x = prevCoords.x;
                markerSizerCoords.width += prevCoords.width;
            }
        }
        else if (msString == 'INCLUDE-FOLLOWING')
        {
            if (markerSizer.nextSibling)
            {
                var followCoords = getComponentPosition(markerSizer.nextSibling, true);
                markerSizerCoords.width += followCoords.width;
            }
        }
        else
            tempSizer = eval(msString);

        if (tempSizer != null)
        {
            var tempCoords = getComponentPosition(tempSizer, true);
            markerSizerCoords.x = tempCoords.x;
            markerSizerCoords.width = tempCoords.width;
        }
    }
    if (dragDropHandler.targetNode.getAttribute('dd_marker_sizer_y') != null)
    {
        var msString = dragDropHandler.targetNode.getAttribute('dd_marker_sizer_y');
        var tempSizer = null;
        if (msString == 'OWNER-TABLE')
        {
            var temp = dragDropHandler.targetNode;
            while (temp != null)
            {
                if (temp.tagName.toLowerCase() == 'table')
                {
                    tempSizer = temp;
                    break;
                }
                temp = temp.parentNode;
            }
        }
        else if (msString == 'INCLUDE-PREVIOUS')
        {
            if (markerSizer.previousSibling)
            {
                var prevCoords = getComponentPosition(markerSizer.previousSibling, true);
                markerSizerCoords.y = prevCoords.x;
                markerSizerCoords.height += prevCoords.height;
            }
        }
        else if (msString == 'INCLUDE-FOLLOWING')
        {
            if (markerSizer.nextSibling)
            {
                var followCoords = getComponentPosition(markerSizer.nextSibling, true);
                markerSizerCoords.height += followCoords.height;
            }
        }
        else
            tempSizer = eval(msString);

        if (tempSizer != null)
        {
            var tempCoords = getComponentPosition(tempSizer, true);
            markerSizerCoords.y = tempCoords.y;
            markerSizerCoords.height = tempCoords.height;
        }
    }

    var markerClass = 'dd_drop_marker_' + dragDropHandler.targetLocation;

    if (dragDropHandler.targetLocation == 'top')
    {
        //param order : top, left, height, width
        dragDropHandler.dropMarker = dragDropHandler.createDropMarkerLine(
                            (elemCoords.y - 5 - dropMarkerOffset) + 'px',
                            markerSizerCoords.x + 'px',
                            '5px',
                            markerSizerCoords.width + 'px',
                            markerClass);
    }
    else if (dragDropHandler.targetLocation == 'bottom')
    {
        //param order : top, left, height, width
        dragDropHandler.dropMarker = dragDropHandler.createDropMarkerLine(
                            (elemCoords.y + elemCoords.height + dropMarkerOffset) + 'px',
                            markerSizerCoords.x + 'px',
                            '5px',
                            markerSizerCoords.width + 'px',
                            markerClass);
    }
    else if (dragDropHandler.targetLocation == 'left')
    {
        //param order : top, left, height, width
        dragDropHandler.dropMarker = dragDropHandler.createDropMarkerLine(
                            (markerSizerCoords.y) + 'px',
                            (elemCoords.x - 3 - dropMarkerOffset) + 'px',
                            (markerSizerCoords.height) + 'px',
                            '5px',
                            markerClass);
    }
    else if (dragDropHandler.targetLocation == 'right')
    {
        //param order : top, left, height, width
        dragDropHandler.dropMarker = dragDropHandler.createDropMarkerLine(
                            (markerSizerCoords.y) + 'px',
                            (elemCoords.x - 1 + elemCoords.width + dropMarkerOffset) + 'px',
                            (markerSizerCoords.height) + 'px',
                            '5px',
                            markerClass);
    }
    else if (dragDropHandler.targetLocation == 'into')
    {
        //param order : top, left, height, width
        dragDropHandler.dropMarker = dragDropHandler.createDropMarkerLine(
                            (elemCoords.y + (elemCoords.height / 2) - 2.5) + 'px',
                            (elemCoords.x + 5) + 'px',
                            '5px',
                            (elemCoords.width - 10) + 'px',
                            markerClass);
    }
    else if (dragDropHandler.targetLocation == 'around')
    {
        //around is a special case, as we actually need to create 4 markers,
        //one for each side of the rectangle.
        //cant just use one div as this would then impact the mouse events, as the mouse would always be over it
        var mkleft = (markerSizerCoords.x - 5 - dropMarkerOffset) + 'px';
        var mkwidth = (markerSizerCoords.width + (dropMarkerOffset * 2) + 10) + 'px';
        var mktop = (markerSizerCoords.y - 5 - dropMarkerOffset) + 'px';
        var mkheight = (markerSizerCoords.height + (dropMarkerOffset * 2) + 10) + 'px';
        var mkbottom = (markerSizerCoords.y + markerSizerCoords.height + dropMarkerOffset) + 'px';
        var mkright = (markerSizerCoords.x + markerSizerCoords.width + dropMarkerOffset) + 'px';

        dragDropHandler.dropMarker = new Object();
        //param order : top, left, height, width
        dragDropHandler.dropMarker.top = dragDropHandler.createDropMarkerLine(
                            mktop, mkleft, '5px', mkwidth, markerClass + '_top');
        dragDropHandler.dropMarker.left = dragDropHandler.createDropMarkerLine(
                            mktop, mkleft, mkheight, '5px', markerClass + '_left');
        dragDropHandler.dropMarker.bottom = dragDropHandler.createDropMarkerLine(
                            mkbottom, mkleft, '5px', mkwidth, markerClass + '_bottom');
        dragDropHandler.dropMarker.right = dragDropHandler.createDropMarkerLine(
                            mktop, mkright, mkheight, '5px', markerClass + '_right');
    }
}

/**
 * Actually creates a line to use as a drop mark
 * @private
 */
dragDropHandler.createDropMarkerLine = function(top, left, height, width, className)
{
    var tempMarker = document.createElement("div");
    tempMarker.style.overflow = "hidden";
    tempMarker.style.background = "black";
    tempMarker.style.position = "absolute";

    tempMarker.style.top = top;
    tempMarker.style.height = height;
    tempMarker.style.left = left;
    tempMarker.style.width = width;

    if (typeof(className) != 'undefined')
        tempMarker.className = className;

    document.body.appendChild(tempMarker);

    WinManager.attachEvent(tempMarker, 'onmousemove', function(e){if (!e) {e = window.event;}
        dragDropHandler.onmousemove(e, dragDropHandler.targetNode, dragDropHandler.targetNode.getAttribute("dnd_handler"));});
    WinManager.attachEvent(tempMarker, 'onmouseup', function(e){if (!e) {e = window.event;}
        dragDropHandler.onmouseup(e, dragDropHandler.targetNode, dragDropHandler.targetNode.getAttribute("dnd_handler"));});

    return tempMarker;


}

/**
 * Checks the type attribute of the drag drop container identified by the given id
 * against that of the currently dragging object.
 * Returns true if they are the same, false otherwise.
 */
dragDropHandler.checkType = function(id)
{
    if (dragDropHandler.currentID != null)
    {
        if (dragDropHandler.targets[dragDropHandler.currentID].type ==
                dragDropHandler.targets[id].type)
        {
            return true;
        }

    }
    return false;
}



dragDropHandler.onmousedown = function(e, elem, id)
{
    dragDropHandler.mode = 'mouseDown';
    dragDropHandler.sourceNode = elem;
    dragDropHandler.currentID = id;
    /*alert(e);
    alert(e.preventDefault + " : " + typeof(e.preventDefault));*/
    if (e.preventDefault) { e.preventDefault(); }
    e.returnValue = false;
}


dragDropHandler.onmouseup = function(e, elem, id)
{
    if (dragDropHandler.mode != 'off')
    {
        dragDropHandler.mode = 'off';
        scrollableDivManager.scrollEnabled = false;

        dragDropHandler.removeMoveNode();
        clearOpacity(dragDropHandler.sourceNode);

        if (typeof(elem) != 'undefined')
        {
            if ((dragDropHandler.targetLocation != null) && (dragDropHandler.sourceNode != null))
            {
                //call method on target container
                if (typeof(id) != 'undefined')
                {
                    var moveAccepted = true;

                    if (dragDropHandler.targets[id].method != null)
                    {
                        var tc = dragDropHandler.targets[id].container;
                        var te = dragDropHandler.targetNode;
                        var sc = dragDropHandler.targets[dragDropHandler.currentID].container;
                        var se = dragDropHandler.sourceNode;
                        var loc = dragDropHandler.targetLocation;
                        moveAccepted = eval(dragDropHandler.targets[id].method + "(sc, se, tc, te, loc, e)");
                    }

                    /*if ((moveAccepted == true) || (moveAccepted == null) || (typeof(moveAccepted) == 'undefined'))
                    {
                        if (dragDropHandler.dropMarker.parentNode != null)
                            dragDropHandler.dropMarker.parentNode.replaceChild(dragDropHandler.sourceNode,
                                                                               dragDropHandler.dropMarker);

                        //if it was dropped into a different container, update the reference
                        if (dragDropHandler.currentID != id)
                        {
                            dragDropHandler.sourceNode.setAttribute('dnd_handler', id);
                        }
                    }*/

                }


                dragDropHandler.sourceNode = null;
                dragDropHandler.targetNode = null;
                dragDropHandler.targetLocation = null;
                dragDropHandler.currentID = null;
            }
        }
        dragDropHandler.removeDropMarker();

        document.onselectstart = null;
    }
}

/**
 * Handler for mouse move events.
 */
dragDropHandler.onmousemove = function(e, elem, id)
{
    //should we start a drag event???
    if (dragDropHandler.mode == 'mouseDown' && typeof(dragDropHandler.sourceNode) != 'undefined')
    {
        //check the container being dragged over provides a drag source
        if ((dragDropHandler.targets[dragDropHandler.currentID].sourceTarget == 'both') || (dragDropHandler.targets[dragDropHandler.currentID].sourceTarget == 'source'))
        {
            dragDropHandler.mode = "moving";
            scrollableDivManager.scrollEnabled = true;
            //create the new move node (this will delete any existing one)
            dragDropHandler.createMoveNode();

            //stop the visual move node from picking up mouse events
            WinManager.detachEvent(dragDropHandler.moveNode, "onmousedown", dndEventHandler);
            WinManager.detachEvent(dragDropHandler.moveNode, "onmouseup", dndEventHandler);
            WinManager.detachEvent(dragDropHandler.moveNode, "onmousemove", dndEventHandler);

            WinManager.attachEvent(dragDropHandler.moveNode, 'onmousemove', function(e){if (!e){ e = window.event;}
                if (dragDropHandler.targetNode == null) dragDropHandler.onmousemove(e);
                else
                dragDropHandler.onmousemove(e, dragDropHandler.targetNode, dragDropHandler.targetNode.getAttribute("dnd_handler"));});
            WinManager.attachEvent(dragDropHandler.moveNode, 'onmouseup', function(e){if (!e) {e = window.event;}
                if (dragDropHandler.targetNode == null) dragDropHandler.onmouseup(e);
                else
                dragDropHandler.onmouseup(e, dragDropHandler.targetNode, dragDropHandler.targetNode.getAttribute("dnd_handler"));});


            dragDropHandler.moveNode.style.position="absolute";
            document.body.appendChild(dragDropHandler.moveNode);

            setOpacity(dragDropHandler.sourceNode, 0.5);
            setOpacity(dragDropHandler.moveNode, 0.5);


            //focus a different field to ensure onchange gets called if needed for the currently
            //selected text field
            dragDropHandler.blurFocussedField();
            // cancel out any text selections
            document.body.focus();
            // prevent text selection in IE
            document.onselectstart = function () { return false; };
            // prevent IE from trying to drag an image
            dragDropHandler.moveNode.ondragstart = function() { return false; };

            if (typeof(dragDropHandler.dragStarted) == 'function')
                dragDropHandler.dragStarted();
        }
    }

    if (dragDropHandler.mode == 'moving')
    {
        moveToMouseCoords(e, dragDropHandler.moveNode);
        /*if (e.preventDefault) { e.preventDefault(); }
        e.returnValue = false
        e.cancelBubble = true;*/
        dragDropHandler.mode = 'processed';

        //if we are currently moving outside of this container, then hide the drop marker
        if (typeof(elem) == 'undefined')
        {
            dragDropHandler.removeDropMarker();
        }
        else  //move the drop marker as needed
        {
            if ((elem.getAttribute('dnd_type') != 'container') && dragDropHandler.checkType(id))
            {
                //check the container being dragged over provides a drag target
                if ((dragDropHandler.targets[id].sourceTarget == 'both') || (dragDropHandler.targets[id].sourceTarget == 'target'))
                {
                    var mouseCoords = getMouseCoords(e);
                    var elemCoords = getComponentPosition(elem, true);

                    //get the set of possible insert locations, given the current target element,
                    //and the drag source object
                    locations = dragDropHandler.getInsertLocations(elem);

                    var newMarkerLocation = dragDropHandler.findNewMarkerLocation(locations, mouseCoords, elemCoords);

                    //dragDropHandler.moveNode.innerHTML = newMarkerLocation;

                    if (newMarkerLocation != null)
                    {
                        if ((elem != dragDropHandler.targetNode) || (dragDropHandler.targetLocation != newMarkerLocation))
                        {
                            //remove the existing marker
                            dragDropHandler.removeDropMarker();

                            dragDropHandler.targetNode = elem;
                            dragDropHandler.targetLocation = newMarkerLocation

                            //create the new marker
                            dragDropHandler.createDropMarker(dragDropHandler.targets[id].dropMarkerOffset);
                        }
                    }
                }
            }
        }
    }

    //if we have reached (event has bubbled up to) the document, and this event has been processed,
    //then reset the mode for the next event.
    if ((typeof(elem) == 'undefined') && (dragDropHandler.mode == 'processed'))
    {
        dragDropHandler.mode = 'moving';
    }
}

/**
 * Attempts to remove the focussed from the selected field (if any)
 * It is not possible to easily work out which field has focus, so instead this
 * temporarily focuses the first field it finds, and then blurs it.
 * This is needed to make sure that any onchange events defined for the currently focussed field
 * are fired if needed before the drag process begins. (eg to store away any changes)
 */
dragDropHandler.blurFocussedField = function()
{
    var newInput = document.createElement("input");
    newInput.type = 'text';
    document.body.appendChild(newInput);
    newInput.focus();
    newInput.blur();
    document.body.removeChild(newInput);
}


/**
 * Determines the set of possible marker locations given that the mouse is currently over the
 * provided element, and knowledge of the drag source element.
 * Only marker locations supported by both elements will be returned.  If an element does not specify
 * the marker possibilities directly, then the default settings for the appropriate container will be used.
 * If the continer also did not specify, then 'top' and 'bottom' will be assumed.
 *
 * @param elem The DOM element that the mouse is moving over.  This will be a drag target element.
 * @return An object containing properties for all the applicable marker locations.
 *              The full set of possible values is: 'top', 'bottom', 'left', 'right', 'around', 'into'
 */
dragDropHandler.getInsertLocations = function(elem)
{
    var targetLocations = dragDropHandler.targets[elem.getAttribute("dnd_handler")].targetLocations;

    if (elem.getAttribute('dd_locations_target') != null)
    {
        options = elem.getAttribute('dd_locations_target').replace(/\s/g, '').split(',');
        targetLocations = new Object();
        for (var i = 0; i < options.length; ++i)
        {
            targetLocations[options[i]] = true;
        }
    }

    var sourceLocations = dragDropHandler.targets[dragDropHandler.currentID].sourceLocations;
    if (dragDropHandler.sourceNode.getAttribute('dd_locations_source') != null)
    {
        options = dragDropHandler.sourceNode.getAttribute('dd_locations_source').replace(/\s/g, '').split(',');
        sourceLocations = new Object();
        for (var i = 0; i < options.length; ++i)
        {
            sourceLocations[options[i]] = true;
        }
    }

    var matchedLocations = new Object();
    for (loc in targetLocations)
    {
        if (sourceLocations[loc])
            matchedLocations[loc] = true;
    }

    return matchedLocations;

}

/**
 * Works out which marker location should be used, given the list of possible locations
 * the current mouse coords, and the coords of the current element.
 * @param locations The set of locations that could possible be used.  This should already have been
 *                  filtered based on the current source and target elements.
 * @param mouseCoords Object holding the current mouse coourdinate details
 * @param elemCoords Object holding the coordinate and size details of the current target element.
 * @return The marker location name that should be used.
 */
dragDropHandler.findNewMarkerLocation = function(locations, mouseCoords, elemCoords)
{
    var elemInner = new Object();
    elemInner.x = elemCoords.x + (elemCoords.width / 4);
    elemInner.y = elemCoords.y + (elemCoords.height / 4);
    elemInner.width = elemCoords.width / 2;
    elemInner.height = elemCoords.height / 2;

    var elemOuter = new Object();
    elemOuter.x = elemCoords.x + (elemCoords.width / 10)
    elemOuter.y = elemCoords.y + (elemCoords.height / 10)
    elemOuter.width = elemCoords.width * 0.8;
    elemOuter.height = elemCoords.height * 0.8;

    var locationValues = dragDropHandler.determineMarkerLocationPriorities(locations, mouseCoords, elemCoords, elemInner, elemOuter);

    //now work out which type got the highest score
    var returnLocation = null;
    var locationVal = 0;
    for (prop in locationValues)
    {
        if (locationValues[prop] > locationVal)
        {
            locationVal = locationValues[prop];
            returnLocation = prop;
        }
    }

    return returnLocation;
}


dragDropHandler.determineMarkerLocationPriorities = function(locations, mouseCoords, elemCoords, elemInner, elemOuter)
{
    var locationValues = new Object();

    if (locations.into)
    {
        locationValues.into = 1;
        //increase score if not in the top or bottom areas
        if ((mouseCoords.y > elemInner.y) && (mouseCoords.y < (elemInner.y + elemInner.height)))
        {
            locationValues.into = 3;
            //increase score if not in the left or right section
            if ((mouseCoords.x > elemInner.x) && (mouseCoords.x < (elemInner.x + elemInner.width)))
            {
                locationValues.into = 5;
            }
        }
    }
    if (locations.top)
    {
        locationValues.top = 1;
        //increase score if in the top half.
        if (mouseCoords.y < (elemCoords.y + (elemCoords.height / 2)))
        {
            locationValues.top = 2;
            //increase score if not in the into area
            if (mouseCoords.y < elemInner.y)
            {
                locationValues.top = 3;
                //increase score if not in the left or right areas
                if ((mouseCoords.x > elemInner.x) && (mouseCoords.x < (elemInner.x + elemInner.width)))
                {
                    locationValues.top = 4;
                    //increase score if not in the around area
                    if (mouseCoords.y > elemOuter.y)
                    {
                        locationValues.top = 5;
                    }
                }
            }
        }
    }
    if (locations.bottom)
    {
        locationValues.bottom = 1;
        //increase score if in the bottom half.
        if (mouseCoords.y > (elemCoords.y + (elemCoords.height / 2)))
        {
            locationValues.bottom = 2;
            //increase score if not in the into area
            if (mouseCoords.y > (elemInner.y + elemInner.height))
            {
                locationValues.bottom = 3;
                //increase score if not in the left or right areas
                if ((mouseCoords.x > elemInner.x) && (mouseCoords.x < (elemInner.x + elemInner.width)))
                {
                    locationValues.bottom = 4;
                    //increase score if not in the around area
                    if (mouseCoords.y < (elemOuter.y + elemOuter.height))
                    {
                        locationValues.bottom = 5;
                    }
                }
            }
        }
    }
    if (locations.left)
    {
        locationValues.left = 1;
        //increase score if in the left half.
        if (mouseCoords.x < (elemCoords.x + (elemCoords.width / 2)))
        {
            locationValues.left = 2;
            //increase score if not in the into area
            if (mouseCoords.x < elemInner.x)
            {
                locationValues.left = 3;
                //increase score if not in the top or bottom areas
                if ((mouseCoords.y > elemInner.y) && (mouseCoords.y < (elemInner.y + elemInner.height)))
                {
                    locationValues.left = 4;
                    //increase score if not in the around area
                    if (mouseCoords.x > elemOuter.x)
                    {
                        locationValues.left = 5;
                    }
                }
            }
        }
    }
    if (locations.right)
    {
        locationValues.right = 1;
        //increase score if in the left half.
        if (mouseCoords.x > (elemCoords.x + (elemCoords.width / 2)))
        {
            locationValues.right = 2;
            //increase score if not in the into area
            if (mouseCoords.x > (elemInner.x + elemInner.width))
            {
                locationValues.right = 3;
                //increase score if not in the top or bottom areas
                if ((mouseCoords.y > elemInner.y) && (mouseCoords.y < (elemInner.y + elemInner.height)))
                {
                    locationValues.right = 4;
                    //increase score if not in the around area
                    if (mouseCoords.x < (elemOuter.x + elemOuter.width))
                    {
                        locationValues.right = 5;
                    }
                }
            }
        }
    }
    if (locations.around)
    {
        locationValues.around = 1;

        //increase score if in the around area
        if ((mouseCoords.x < elemOuter.x) || (mouseCoords.x > (elemOuter.x + elemOuter.width)) || //left or right
            (mouseCoords.y < elemOuter.y) || (mouseCoords.y > (elemOuter.y + elemOuter.height))) //top or bottom
        {
            locationValues.around = 5;
        }
    }

    return locationValues;
}



/**
 * Generic event handler that gets attached as required, and just delegates to the specific method
 * on the dragDropHandler instance
 */
function dndEventHandler(e)
{
    if (!e)
    {
        e = window.event;
    }

    var source = (e.target) ? e.target : e.srcElement;
    if (source.nodeType == 3) // defeat Safari bug
        source = source.parentNode;

    var tn = source.tagName.toLowerCase();

    //stop the event being processed if it would start a drag on an input type control
    //eg want to still be able to click into text boxes
    if (((e.type != 'mousemove') || (dragDropHandler.mode != 'moving')) &&
        !((e.type == 'mouseup') && (dragDropHandler.mode == 'moving')) &&
        ((tn == 'a') || (tn == 'input') || (tn == 'select') || (tn == 'textarea') || (tn == 'button')))
    {
        //provide the ability to specifically indicate that an element can start a drag
        if (source.className.indexOf('allow-dd-events') == -1)
            return;
    }

    //work up the HTML dom to try and find the element that can actually be the drag source
    while (source != null)
    {
        if (source.nodeType == 9) //reached the document node so cant drag
        {
            source = null;
            break;
        }

        if (source.getAttribute("dnd_handler") != null)
        {
            //found an element that has a drag handler, but need to check if it can provide a drag source
            if ((dragDropHandler.mode != "moving") && (source.getAttribute("dd_locations_source") != null) && (source.getAttribute("dd_locations_source") == ''))
            {
                //this cant be a source, so keep searching
            }
            else
                break;  //source found
        }

        source = source.parentNode;
    }

    if (source != null)
    {
        //cant drag the containers
        if (source.getAttribute('dnd_type') == 'container')
            return;

        eval("dragDropHandler.on" + e.type + "(e, source, " + source.getAttribute("dnd_handler") + ")");

        return false;
    }
}





/**
 * DragDropContainer is used to store the details of each different container used for drag and drop
 *
 * @param container The HTML component
 * @param type The type of this container.  can only drag between containers of the same type.
 * @param method The name of the method to call after completing a drag and drop operation into this container.
 * @param relevantClass (Optional) The class name that draggable components must have
 * @param sourceTarget (Optional) indicates whether this container is only a source or target for drag operations.
 *                          Possible values - 'source', 'target', 'both' (default is both)
 * @param dropMarkerOffset (Optional) The offset in pixels to draw the drop marker relative to the target location
 * @param sourceLocations (Optional) An array of properties indicating which drag locations can be used for elements
 *                          being dragged from this container. eg {top:true, bottom:true}
 *                          The full set of possible properties is: 'top', 'bottom', 'left', 'right', 'around', 'into'
 * @param targetLocations (Optional) An array of properties indicating which drag locations can be used for elements
 *                          being dragged into this container.
 */
function dragDropContainer(container, type, method, relevantClass, sourceTarget, dropMarkerOffset,
                            sourceLocations, targetLocations)
{
    this.container = container;
    this.type = type;
    this.method = method;
    this.relevantClass = relevantClass;
    this.sourceTarget = sourceTarget;
    this.dropMarkerOffset = dropMarkerOffset;

    if (this.method == '')
        this.method = null;

    if ((this.sourceTarget == '') || (typeof(this.sourceTarget) == 'undefined'))
        this.sourceTarget = 'both';

    if ((this.dropMarkerOffset == '') || (typeof(this.dropMarkerOffset) == 'undefined'))
        this.dropMarkerOffset = 0;

    this.sourceLocations = sourceLocations;
    if ((sourceLocations == '') || (typeof(sourceLocations) == 'undefined'))
        this.sourceLocations = dragDropHandler.DEFAULT_MARKER_LOCATIONS;

    this.targetLocations = targetLocations;
    if ((targetLocations == '') || (typeof(targetLocations) == 'undefined'))
        this.targetLocations = dragDropHandler.DEFAULT_MARKER_LOCATIONS;
}



//---------------------- Utility Functions ---------------------------------------------


/**
 * Moves the given object to the coordinates of the given event
 */
function moveToMouseCoords(evt, obj)
{
    var coords = getMouseCoords(evt);
    obj.style.top = (coords.y + 5) + 'px';
    obj.style.left = (coords.x + 5) + 'px';
}


/**
 * Returns all the elements that are children of the given obj (at any level)
 * that have the given class name applied.
 * @return An array of matching elements
 */
function getElementsByClassName(obj, className)
{
    var matches = [];
    if ((obj == null) || (obj.nodeType != 1))
        return matches

    for (var i = 0; i < obj.childNodes.length; ++i)
    {
        var child = obj.childNodes.item(i);
        if (child.nodeType == 1)
        {
            //check if this elemnt matches
            if (child.className.indexOf(className) != -1)
                matches[matches.length] = child;

            //recursively process any children
            matches = matches.concat(getElementsByClassName(child, className));

        }
    }

    //return the matches
    return matches;
}




//call the initiate function
dragDropHandler.initiate();