/**
* Handler object
* Encapsulates a handler in the formmaker application XML, provides utility methods to
* output SVG and Formmaker XML.
*
* @param name Javascript reference name, needed when added callbacks in SVG output
* @param container The parent object of this page
* @param data Optional Data in XML or format to initialise the Handler, if not provided must set x, y, id separately
*/
function Handler(name, container, data)
{
    this._name = name;
    this._container = container;
    this._mousedown = false;

    this._defaults = 'hidden';
    this._incomingLinks = new Array();
    this._outgoingLinks = new Array();
    this._node = "";
    this._nodeID = "";
    //this._nodePath = "";
    this._nodePath = this._container._projectInfo;
    this._pattern = "";
    this._type = "";
    this._impl = "";
    this._location = "";

    this._hasChanges = false;

    if(typeof(data) != 'undefined')
    {
        this._id = data.getAttribute('id');
        this._x = Number(data.getAttribute('x'));
        this._y = Number(data.getAttribute('y'));

        //checks whether this is a normal handler or subsection handler.
        //will assign a type if one doesn't exist as of yet.
        this._type = data.getAttribute('type');
        if(this._type == null || this._type == "null" || this._type =="")
            this._type = "normal";

        this._impl = data.getAttribute('implementation');
        if(this._impl == null || this._impl == "null" || this._impl =="")
        {
            if (userAppType == 'java')
                this._impl = 'java';
            else
                this._impl = 'xml';
        }

        var nodeNode = data.selectSingleNode("fm:node");
        if(nodeNode != null)
        {
            this._node = getElementText(nodeNode);
            var nodeArray = this._node.split('-');
            this._nodeID = nodeArray[nodeArray.length - 1];
            this._pattern = nodeArray[nodeArray.length - 2];
        }
    }
    else
    {
        this._id = '';
        this._x = 0;
        this._y = 0;
        if(container._mode == "add_subsection")
            this._type = "subsection";
        else if(container._mode == "add_thirdPartyService")
            this._type = "thirdparty";
        else
            this._type = "normal";

        if (userAppType == 'java')
            this._impl = 'java';
        else
            this._impl = 'xml';
    }

    if(this._type == "thirdparty")
    {
        if(data != undefined && data != null && data.selectSingleNode("fm:URL") != null)
        {
            this._location = getElementText(data.selectSingleNode("fm:URL"));
        }
    }

    if(this._type == "normal")
    {
        //this._imagePath = "../images/node.png";
        this._imagePath = "#controller";
        this._imageWidth = 10;
        this._imageHeight = 10;
    }
    else if(this._type == "thirdparty")
    {
        //this._imagePath = "../images/thirdParty.png";
        this._imagePath = "#thirdparty";
        this._imageWidth = 10;
        this._imageHeight = 10;
    }

    /*
    * Setup class methods
    */
    if (typeof(__HandlerPrototypeCalled) == 'undefined')
    {
        __HandlerPrototypeCalled = true;
        Handler.prototype.draw = draw;
        Handler.prototype.redraw = redraw;
        Handler.prototype.getJavascriptIdentifier = getJavascriptIdentifier;
        Handler.prototype.onmousedown = onmousedown;
        Handler.prototype.onclick = onclick;
        Handler.prototype.onmousemove = onmousemove;
        Handler.prototype.onmouseup = onmouseup;
        Handler.prototype.addHighlight = addHighlight;
        Handler.prototype.removeHighlight = removeHighlight;
        Handler.prototype.remove = remove;
        Handler.prototype.rename = rename;
        Handler.prototype.processNewName = processNewName;
        Handler.prototype.addIncomingLink = addIncomingLink;
        Handler.prototype.removeIncomingLink = removeIncomingLink;
        Handler.prototype.addOutgoingLink = addOutgoingLink;
        Handler.prototype.removeOutgoingLink = removeOutgoingLink;
        Handler.prototype.getNodeInfo = getNodeInfo;
        Handler.prototype.getApplicationXML = getApplicationXML;
        Handler.prototype.hideServices = hideServices;
        Handler.prototype.displayContextPanel = displayContextPanel;
        Handler.prototype.viewControllerRules = viewControllerRules;
        Handler.prototype.viewJavaCode = viewJavaCode;
        Handler.prototype.togglePageDefaults = togglePageDefaults;
        Handler.prototype.implTypeChanged = implTypeChanged;
        Handler.prototype.updateTargetPageDataSources = updateTargetPageDataSources;

        Handler.prototype.setChanged = setChanged;
        Handler.prototype.clearChanged = clearChanged;
    }

    /**
    * Adds an incoming link to this handler
    */
    function addIncomingLink(link)
    {
        this._incomingLinks.push(link);
        this.setChanged();
    }

    /**
    * Removes an incoming link from this handler
    */
    function removeIncomingLink(link)
    {
        this._incomingLinks = removeElement(link, this._incomingLinks);
        for(var i = 0; i < this._incomingLinks.length; ++i)
        {
            if(this._incomingLinks[i] != undefined)
            {
                this._incomingLinks[i].redraw();
            }
        }
        this.setChanged();
    }

    /**
    * Adds an outgoing link to this handler
    */
    function addOutgoingLink(link)
    {
        this._outgoingLinks.push(link);
        //this.setChanged();
    }

    /**
    * Removes an outgoing link from this handler
    */
    function removeOutgoingLink(link)
    {
        this._outgoingLinks = removeElement(link, this._outgoingLinks);
        for(var i = 0; i < this._outgoingLinks.length; ++i)
        {
            if(this._outgoingLinks[i] != undefined)
            {
                this._outgoingLinks[i].redraw();
            }
        }
        this.setChanged();
    }

    /**
    * Create SVG for the Handler and attach it to the containers SVG context
    */
    function draw()
    {
        SVGDocument = this._container.getSVGDocument();
        SVGContext = this._container.getSVGNodesContext();

        //Image
        /*var dummy = SVGDocument.getElementById('dummy_image');
        var image = dummy.cloneNode(true);*/
        var image = SVGDocument.createElementNS(SVG_NS, 'use');
        image.setAttribute('id', "handler_" + this._id);
        image.setAttribute('x', - Number(this._imageWidth/2));
        image.setAttribute('y', - Number(this._imageHeight/2));
        image.setAttribute('width', this._imageWidth);
        image.setAttribute('height', this._imageHeight);
        //image.setAttribute('display', 'inline');
        //image.setAttribute('xlink:href', this._imagePath);
        image.setAttributeNS(XLINK_NS, 'xlink:href', this._imagePath);

        //Text
        var text = SVGDocument.createElementNS(SVG_NS, 'text');
        var textData = SVGDocument.createTextNode(this._id);
        text.appendChild(textData);
        text.setAttribute('x', 0);
        text.setAttribute('y', 10);
        text.setAttribute('text-anchor', 'middle');
        text.setAttribute('pointer-events', 'none');
        text.setAttribute('style', 'font-size: 5px');

        //Group
        var group = SVGDocument.createElementNS(SVG_NS, 'g');
        group.setAttribute('id', this._id);
        group.setAttribute('transform', 'translate(' + this._x  + ', ' + this._y + ')');
        group.setAttribute('onmousemove', 'window.parent.' + this.getJavascriptIdentifier() + '.onmousemove(evt)' );
        group.setAttribute('onmouseup', "handleMouseUp('" +this.getJavascriptIdentifier() + "', evt)" );
        group.setAttribute('onmousedown', "handleMouseDown('" +this.getJavascriptIdentifier() + "', evt)" );
        group.setAttribute('onclick', "handleMouseClick('" +this.getJavascriptIdentifier() + "', evt)" );
        group.appendChild(image);
        group.appendChild(text);
        SVGContext.appendChild(group);

        //Maintain reference to group
        this._group = group;

        //Draw child links
        for(var i = 0; i < this._outgoingLinks.length; ++i)
        {
            if(this._outgoingLinks[i] != undefined)
            {
                this._outgoingLinks[i].redraw();
            }
        }

        //check if this handler should still be highlighted
        if (this._container._selected == this)
        {
            this.addHighlight();
        }
    }

    /**
    * Delete old SVG for the handler then redraw it
    */
    function redraw()
    {
        this._container.getSVGNodesContext().removeChild(this._group);
        this.draw();
    }

    /**
    * Mouse down event handler for this object
    */
    function onmousedown(event)
    {
        var mode = this._container._mode;

        switch(mode)
        {
            case 'select' :
                this._container._moving = this;
            var screenCoords = new Object();
            screenCoords.x = event.clientX;
            screenCoords.y = event.clientY;
            this._mdposition = this._container._svgCoordinates.getUserSpaceCoordinates(screenCoords);
            this._mousedown = true;
            break;

            /* The following has been commented to ensure the last target component does not act as the start of of the link by default. */
            /*case 'add_link':
                var linkID = this._id + '__temp__link';
                this._container._tempLink = new Link(linkID, null, this._container, this);
          this._container._tempLink.draw();
          this._container._tempLink._moving = true;
          break;*/
        }
    }

    /**
    * Mouse move event handler for this object
    */
    function onmousemove(event)
    {
        switch(this._container._mode)
        {
            case 'select':
                if(this._mousedown == true)
                {
                    this._container._svgCoordinates.reset();
                    this._moved = true;
                    var xdiff = this._container._svgCoordinates.getUserSpaceX(event.clientX);
                    var ydiff = this._container._svgCoordinates.getUserSpaceY(event.clientY);
                    var newPos = new Object();
                    newPos.x = xdiff;
                    newPos.y = ydiff;

                    if (differentMousePosition(this._mdposition, newPos) == true)
                    {
                          this._x = xdiff;
                          this._y = ydiff;
                          this._mdposition = null;

                          /*var transform = "translate(" + xdiff + ", " + ydiff + ")";
                          this._group.setAttribute('transform', transform);*/
                          this.redraw();

                          for(var i = 0; i < this._incomingLinks.length; ++i)
                          {
                              if(this._incomingLinks[i] != undefined)
                              {
                                  this._incomingLinks[i].redraw();
                              }
                          }
                    }
                }
                break;

            case 'add_link':
                if(typeof(this._container._tempLink) != 'undefined')
                {
                    this._container._tempLink.onmousemove(event);
                }
                break;
        }
    }

    /**
    * Mouse up event handler
    */
    function onmouseup(event)
    {
        this._mousedown = false;
        this._container._moving = undefined;
        if(this._moved == true)
        {
            this._moved = false;
            var newx = this._container._svgCoordinates.getUserSpaceX(event.clientX);
            var newy = this._container._svgCoordinates.getUserSpaceY(event.clientY);
            var newPos = new Object();
            newPos.x = newx;
            newPos.y = newy;

            if (differentMousePosition(this._mdposition, newPos) == true)
            {
                this._x = newx;
                this._y = newy;
                this.redraw();
                this.setChanged();
            }
        }

        if(typeof(this._container._tempLink) != 'undefined')
        {
            var tempLink = this._container._tempLink;
            if(this._type=='thirdparty' && tempLink._source.constructor == Handler && tempLink._source._type == 'normal')
            {
                alert("Adding a link from a controller to a third party service is not allowed.");
                tempLink = null;
                this._container._tempLink.remove(this._container._tempLink);
            }
            else if(this._type=='thirdparty' && tempLink._source.constructor == Handler && tempLink._source._type == 'thirdparty')
            {
                alert("Adding a link from a third party service to a third party service is not allowed.");
                tempLink = null;
                this._container._tempLink.remove(this._container._tempLink);
            }
            else if(this._type=='normal' && tempLink._source.constructor == Handler && tempLink._source._type == 'thirdparty')
            {
                alert("Adding a link from a third party service to a controller is not allowed.");
                tempLink = null;
                this._container._tempLink.remove(this._container._tempLink);
            }
            else if(this != tempLink._source)
            {
                if(tempLink._source.constructor == Page)
                {
                    tempLink._painterNode = this._container._defaults._painterNode;
                    var nodeArray = tempLink._painterNode.split('-');
                    tempLink._painterNodeID = nodeArray[nodeArray.length - 1];
                    tempLink._painterPattern = nodeArray[nodeArray.length - 2];
                }
                tempLink._target = this;
                this.addIncomingLink(tempLink);
                findUniqueLinkID(tempLink);
                this._container._links[tempLink._id] = tempLink;
                tempLink._moving = false;
                this.redraw();
                tempLink._source.redraw();

                this._container.storeActionAddition(tempLink._action);
            }
            else
            {
                alert("Adding a link from a controller to itself is not allowed.");
                tempLink = null;
                this._container._tempLink.remove(this._container._tempLink);
            }
            if (this._container._dblclick == false)
                this._container.setMode('select');
            this._container._tempLink = undefined;
        }
        //Bubble mouse up to ensure drag and drop works.
        document.onmouseup();
    }

    /**
    * Onclick event handler for this object
    */
    function onclick(event)
    {
        var mode = this._container._mode;
        switch(mode)
        {
            case 'select' :
                if (this._container._selected != this)
                {
                    //blank out the list of used services
                    selectedNode_nodeList = null;
                    this._container.setSelected(this);
                }
                break;
            case 'delete' :
                this.remove();
                if (this._container._dblclick == false)
                    this._container.setMode('select');
                break;
            case 'rename' :
                this.rename();
                if (this._container._dblclick == false)
                    this._container.setMode('select');
                break;
        }
        if (event.detail == 2)
            this.rename();
    }

    /**
    * Gets a javascript string that identifies this object
    *
    * @returns A string containing javascript that identifies this object
    */
    function getJavascriptIdentifier()
    {
        return this._container.getHandlersIdentifier() + "." + this._name;
    }

    /**
    * Highlights this Handler in the SVG
    */
    function addHighlight()
    {
        SVGDocument = this._container.getSVGDocument();
        var rect = SVGDocument.createElementNS(SVG_NS, 'rect');
        rect.setAttribute('stroke-width', 1);
        rect.setAttribute('opacity', 0.6);
        rect.setAttribute('fill', 'none');
        rect.setAttribute('stroke', 'red');
        rect.setAttribute('x', -5);
        rect.setAttribute('y', -5);
        rect.setAttribute('height', 10);
        rect.setAttribute('width', 10 );
        rect.setAttribute('id', 'highlight');

        this._group.appendChild(rect);
    }

    /**
    * Removes the highlight on this Handler in the SVG
    */
    function removeHighlight()
    {
        for(var i = 0; i < this._group.childNodes.length; ++i)
        {
            if(this._group.childNodes.item(i).getAttribute('id') == 'highlight')
            {
                this._group.removeChild(this._group.childNodes.item(i));
            }
        }
    }

    /**
    * Removes this Handler from the diagram
    */
    function remove()
    {
        showStatusMessage('Please wait...');
        dojo.xhrPost(
        {
            url: "FormMaker",
            handleAs: "text",
            preventCache: "true",
            content:
            {
                xml_project_info: document.result_form.xml_project_info.value,
                action: 'process',
                operation1: 'delete_component',
                pattern_id: this._pattern
            },
            handlerDetails:
            {
                id: this._id
            },
            error: function(response, ioArgs)
            {
                alert('An error occurred while trying to load the information needed to show the appropriate document.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + response.message);
                hideStatusMessage();
                return response;
            },
            load: function(response, ioArgs)
            {
                var dojoResponse = getXMLDocument(response);
                Sarissa.setXpathNamespaces(dojoResponse, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker'");
                nodeCheck = dojoResponse.selectSingleNode("/xfact:outcome/xfact:successful[.='true']");
                //check to see if it was successfully created
                //for now always delete the stuff from the app map regardless of whether deleting the controller on the
                //server actually completed.  REally we should distinguish 'not found' type errors from the others, and act
                //accordingly.
                /*if (nodeCheck != null || navigationDiagram._handlers[ioArgs.args.handlerDetails.id]._type== "thirdparty")
                {*/
                    var currentHandler = navigationDiagram._handlers[ioArgs.args.handlerDetails.id];
                    if(currentHandler._type!='subsection')
                    {
                        navigationDiagram.getSVGNodesContext().removeChild(currentHandler._group);
                    }
                    navigationDiagram.setSelected(undefined);
                    delete navigationDiagram._handlers[ioArgs.args.handlerDetails.id];

                    currentHandler._container.setChanged();

                    //updated the generated data sources lsit for any target pages to remove this
                    //controller from the list
                    currentHandler.updateTargetPageDataSources('delete');


                    //Need to make a copy of the links collections before deleting
                    //because the remove method that gets called updates the collection
                    //resulting in iteration problems

                    var tempIncomingLinks = cloneArray(currentHandler._incomingLinks);

                    for(var i = 0; i < tempIncomingLinks.length; ++i)
                    {
                        if(tempIncomingLinks[i] != undefined)
                        {
                            tempIncomingLinks[i].remove(false);
                        }
                    }

                    var tempOutgoingLinks = cloneArray(currentHandler._outgoingLinks);

                    //Remove all links associated with this page
                    for(var i = 0; i < tempOutgoingLinks.length; ++i)
                    {
                        if(tempOutgoingLinks[i] != undefined)
                        {
                            tempOutgoingLinks[i].remove(false);
                        }
                    }
                    controllerNodeList = null;
                /*}
                else
                {
                    alert('We were unable to delete your controller.');
                }*/
                hideStatusMessage();
                return response;
            }
        });

       /* if(this._type!='subsection')
        {
            this._container.getSVGNodesContext().removeChild(this._group);
        }
        this._container.setSelected(undefined);
        delete this._container._handlers[this._name];

        //Need to make a copy of the links collections before deleting
        //because the remove method that gets called updates the collection
        //resulting in iteration problems

        var tempIncomingLinks = cloneArray(this._incomingLinks);

        for(var i = 0; i < tempIncomingLinks.length; ++i)
        {
            if(tempIncomingLinks[i] != undefined)
            {
                tempIncomingLinks[i].remove(false);
            }
        }

        var tempOutgoingLinks = cloneArray(this._outgoingLinks);

        //Remove all links associated with this page
        for(var i = 0; i < tempOutgoingLinks.length; ++i)
        {
            if(tempOutgoingLinks[i] != undefined)
            {
                tempOutgoingLinks[i].remove(false);
            }
        }*/
    }

    /**
    * Enables inplace renaming of a handlers.
    */
    function rename()
    {
        var old_id = this._id
        var svgCoords = {x: this._x, y: this._y + 6}
        var screenCoords = this._container._svgCoordinates.getScreenCoordinates(svgCoords)
        /* The 'this' param the call back when a new name has been captured. */
        inplaceRenameCapture(screenCoords.x, screenCoords.y, old_id, this);
    }

    /**
    * This is the callback function from the inplaceRenameCapture function after a new name has been entered.
    * @param id The new name.
    */
    function processNewName(id)
    {
        var inputchek = false;
        var old_id = this._id;
        var old_attached_pattern = this._pattern;

        /* Only start doing anything if the captured name is valid and no duplicate exists in the app map. */
        inputchek = ((checkInvalidNames(id)) || (checkInvalidChars(id)) || (checkPropertyNameExists(id, this._container._handlers)));
        if (inputchek == false)
        {
            showStatusMessage('Please wait...');

            var postContent =
                    {
                        xml_project_info: document.result_form.xml_project_info.value,
                        action: 'process',
                        operation1: 'rename_component',
                        pattern_id: old_attached_pattern,
                        new_name: id
                    }

            if (this._impl == 'java')
            {
                postContent.java_file = this._container._defaults._javaCode._location + "/" +
                                        this._container._defaults._javaCode._package.replace(/\./g, '/') +
                                        "/" + this._id.charAt(0).toUpperCase() +
                                        this._id.substring(1) + ".java";
                postContent.java_package = this._container._defaults._javaCode._package;
                postContent.node_id = this._nodeID;

            }

            dojo.xhrPost(
            {
                url: "FormMaker",
                handleAs: "text",
                preventCache: "true",
                content: postContent,
                handlerDetails:
                {
                    old_id: old_id,
                    new_id: id
                },
                error: function(response, ioArgs)
                {
                    alert('An error occurred while trying to rename the controller.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + response.message);
                    hideStatusMessage();
                    return response;
                },
                load: function(response, ioArgs)
                {
                    var dojoResponse = getXMLDocument(response);
                    Sarissa.setXpathNamespaces(dojoResponse, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker'");
                    nodeCheck = dojoResponse.selectSingleNode("/xfact:outcome/xfact:successful[.='true']");
                    //check to see if call was successful
                    if (nodeCheck != null || navigationDiagram._handlers[ioArgs.args.handlerDetails.old_id]._type== "thirdparty")
                    {
                        var current_handler = navigationDiagram._handlers[ioArgs.args.handlerDetails.old_id];
                        delete navigationDiagram._handlers[ioArgs.args.handlerDetails.old_id];
                        current_handler._id = ioArgs.args.handlerDetails.new_id;
                        current_handler._name = ioArgs.args.handlerDetails.new_id;
                        navigationDiagram._handlers[current_handler._name] = current_handler;
                        if(current_handler._type != 'subsection')
                        {
                            current_handler.redraw();
                        }
                        current_handler.getNodeInfo(ioArgs.args.handlerDetails.new_id + '-Controller');
                        controllerNodeList = null;
                        //update the data source generated list for each target page to contain the new controller
                        //name where needed
                        current_handler.updateTargetPageDataSources('rename', ioArgs.args.handlerDetails.old_id);

                        current_handler.setChanged();

                        if(current_handler._type == 'subsection')
                        {
                            navigationDiagram.setSelected(navigationDiagram._pages[current_handler._id]);
                        }
                        else
                        {
                            navigationDiagram.setSelected(current_handler);
                        }
                    }
                    else
                    {
                        /* Server duplicate found - try again. */
                        alert('The name is already in use. Please choose another.');
                        var current_handler = navigationDiagram._handlers[ioArgs.args.handlerDetails.old_id];
                        current_handler.rename();
                    }
                    hideStatusMessage();
                    return response;
                }
            });
            return true;
        }
        else
        {
            /*	Invalid name or client duplicate found - try again. Note, the duplicate alert msg will already have been output by checkPropertyNameExists.
                Calling rename() here causes onblur to be constantly raised in the fmInplaceRenameUpdate. */
            //this.rename();
            return false;
        }
    }

    /**
     * Checks all the outgoing links from this handler to see if
     * they are going to a page.  If so, it checks the data source information for the page.
     * If this details that it has been generated for this controller, then the details are updated
     * accordingly following the remove or delete of this controller.
     * @param mode 'rename' or 'delete'
     * @param old_name If following a rename, this should contain the old name of this controller.
     */
    function updateTargetPageDataSources(mode, old_name)
    {
        var pagesToCheck = new Array();
        if (this._type == 'subsection')
        {
            pagesToCheck[pagesToCheck.length] = this._container._pages[this._id];
        }
        else
        {
            for(var i = 0; i < this._outgoingLinks.length; ++i)
            {
                if(this._outgoingLinks[i] != undefined)
                {
                    if (this._outgoingLinks[i]._target.constructor == Page)
                    {
                        pagesToCheck[pagesToCheck.length] = this._outgoingLinks[i]._target;
                    }
                }
            }
        }

        for (var j = 0; j < pagesToCheck.length; ++j)
        {
            if (pagesToCheck[j] != null)
            {
                var sourcesXML = pagesToCheck[j]._dataSourcesXML;
                if (sourcesXML != '')
                {
                    var tempDoc = getXMLDocument(sourcesXML);
                    Sarissa.setXpathNamespaces(tempDoc, "xmlns:fm='http://www.hyfinity.com/formmaker'");
                    var ds = tempDoc.selectNodes("/fm:data_sources/fm:data_source");
                    for (var k = 0; k < ds.length; ++k)
                    {
                        var generatedString = ds.item(k).getAttribute("generated");
                        generatedString = ',' + generatedString + ',';
                        if (mode == 'rename')
                        {
                            generatedString = generatedString.replace(new RegExp(","+old_name+",", "g"), "," + this._id + ",");
                        }
                        else
                        {
                            generatedString = generatedString.replace(new RegExp(","+this._id+",", "g"), ",");
                        }
                        generatedString = generatedString.substring(1, generatedString.length - 1);
                        ds.item(k).setAttribute("generated", generatedString);
                    }
                    pagesToCheck[j]._dataSourcesXML = new XMLSerializer().serializeToString(tempDoc);
                }
            }
        }
    }


    function getNodeInfo(nodeName)
    {
        if (nodeName == "")
        {
            this._node = "";
            this._nodeID = "";
            this._pattern = "";
        }
        else
        {
            var nodeArray = nodeName.split('-');
            this._pattern = nodeArray[0];
            this._nodeID = nodeArray[1];
            this._node = this._nodePath + '-' + this._pattern + '-' + this._nodeID;
        }
        this.setChanged();
    }

    /**
    * Serializes the state of the diagram to application XML (as a string)
    */
  function getApplicationXML()
  {
      var xml = "<handler xmlns='http://www.hyfinity.com/formmaker' ";
      xml += "id='" + this._id + "' ";
      xml += "x='" + this._x + "' ";
      xml += "y='" + this._y + "' ";
      xml += "type='" + this._type + "' ";
      if(this._type=="thirdparty")
      {
          xml += "><URL>" + this._location + "</URL>";
      }
      else
      {
          xml += 'implementation="'+this._impl+'">';
          xml += "<node>" + this._node + "</node>";
      }
      xml += "</handler>";

      return xml;
    }

    function hideServices()
    {
        hideComponent(document.getElementById("used_services_table"));
        hideComponent(document.getElementById("services_table"));
        hideComponent(document.getElementById("submitButtonsHandlerDIV"));
        hideComponent(document.getElementById("view_rules_button"));
    }

    //status parameter just specifies if the handler has just been created or not
    function displayContextPanel(status)
    {
        var transform = (this._type == 'thirdparty') ? this._container._thirdPartyTransform : this._container._handlerTransform;

        transform.addParameter('handler_javascript_identifier', this.getJavascriptIdentifier());
        transform.addParameter('handler_id', this._id);

        if (this._type != 'thirdparty')
        {
            if (nodeChecker(this._node, 'Controller') == 'false')
                this.getNodeInfo("");
        }

        transform.addParameter('node_id', this._nodeID);
        transform.addParameter('pattern_id', this._pattern);
        //this._container._handlerTransform.addParameter('controller_node_list', controllerNodeList);
        //this._container._handlerTransform.addParameter('model_node_list', modelNodeList);

        if (selectedNode_nodeList == null)
        {
            this._node = "";
            this._nodeID = "";
            this._pattern = "";
            selectedNode_nodeList = getXMLDocument("<empty/>");
        }
        //this._container._handlerTransform.addParameter('selected_node__node_list', selectedNode_nodeList);

        transform.addParameter('user_app_type', userAppType);

        var xml = '<container><model_nodes>' +
                    xmlToString(modelNodeList) +
                    '</model_nodes><controller_nodes>' +
                    xmlToString(controllerNodeList) +
                    '</controller_nodes><selected_nodes>' +
                    xmlToString(selectedNode_nodeList) +
                    '</selected_nodes><application>' +
                    this._container.getApplicationXML() +
                    '</application></container>';

        transform.transform(getXMLDocument(xml));

        if ((this._type != 'thirdparty') && (this._impl != 'java'))
        {
            if (this._defaults == 'hidden')
            {
                if(this._node!='' || status == 'new')
                {
                    this._defaults = 'visible';
                }
            }
            else
                this._defaults = 'hidden';
            this.togglePageDefaults();

            if (document.result_form.controllers_list == null || document.result_form.controllers_list.value == "")
                this.hideServices();
        }
    }

    /**
    * Shows RuleMaker for the selected handlers controller
    */
    function viewControllerRules()
    {
        if (this._impl == 'xml')
        {
            if (this._nodeID != "")
            {
                document.rm_form.node_id.value = this._nodeID;
                document.rm_form.pattern_id.value = this._pattern;

                //create the state document to automatically show an appropriate input document.
                var rm_state = '<rulebase_state xmlns="http://www.hyfinity.com/xengine"><instances>';

                var icl = this._incomingLinks;
                if (this._type == 'subsection')
                    icl = this._container._pages[this._id]._incomingLinks

                //find the first incoming link with a binding document defined.
                for(var i = 0; i < icl.length; ++i)
                {
                    var lk = icl[i]
                    if ((typeof(lk) != 'undefined') &&
                        (lk._source.constructor == Page) &&
                        (lk._instanceType == 'static') &&
                        (lk._dataFromForm._value != ''))
                    {
                        rm_state += '<input visible="true" xstore="' + lk._dataFromForm._xstore + '"';
                        rm_state += ' collection="' + lk._dataFromForm._collection + '"';
                        rm_state += ' document="' + lk._dataFromForm._value + '"'
                        rm_state += ' import_namespaces="true"/>';
                        break;
                    }
                }

                var instanceForm = null
                if (this._type == 'subsection')
                {
                    for(formName in this._container._pages[this._id]._forms)
                    {
                        instanceForm = this._container._pages[this._id]._forms[formName];
                        break;
                    }
                }
                else
                {
                    //find the first outgoing link to a page with a binding document defined.
                    for(var i = 0; i < this._outgoingLinks.length; ++i)
                    {
                        var lk = this._outgoingLinks[i]
                        if ((typeof(lk) != 'undefined') &&
                            (lk._target.constructor == Page))
                        {
                            var form = null;
                            for(formName in lk._target._forms)
                            {
                                form = lk._target._forms[formName];
                                break;
                            }

                            if ((form != null) && (form._dataToForm._value != ''))
                            {
                                instanceForm = form;
                                break;
                            }
                        }
                    }
                }

                if (instanceForm != null)
                {
                    rm_state += '<output visible="true" xstore="' + instanceForm._dataToForm._xstore + '"';
                    rm_state += ' collection="' + instanceForm._dataToForm._collection + '"';
                    rm_state += ' document="' + instanceForm._dataToForm._value + '"'
                    rm_state += ' import_namespaces="true"/>';
                }

                rm_state += '</instances></rulebase_state>';

                document.rm_form.xml_state.value = rm_state;

                //document.rm_form.submit();
                WinManager.resolveFormTargetStart(document.rm_form, "RuleMaker", fm_project, this._pattern, this._nodeID);

                WinManager.cancelPropagation(arguments.callee.caller.arguments[0]);
            }
            else
            {
                alert('Please select a Controller for this Handler before using this feature.');
            }
        }
        else
        {
            alert('This option only applies to XML implementation Controllers.');
        }
    }

    function viewJavaCode()
    {
        if (this._impl == 'java')
        {

            showStatusMessage('Loading...');

            var requestContent;

            //check if a save is needed first

            var currentXML = this._container.getApplicationXML();

            //load the current and previous XML into temporary DOMs.
            var cxd = getXMLDocument(currentXML);
            var pxd = getXMLIsland('applicationXML')
            Sarissa.setXpathNamespaces(cxd, "xmlns:fm='http://www.hyfinity.com/formmaker'");
            Sarissa.setXpathNamespaces(pxd, "xmlns:fm='http://www.hyfinity.com/formmaker'");

            var jLocationSame = (cxd.selectSingleNode("/fm:application/fm:defaults/fm:java_code/fm:location") != null &&
                                 pxd.selectSingleNode("/fm:application/fm:defaults/fm:java_code/fm:location") != null &&
                                 (getElementText(cxd.selectSingleNode("/fm:application/fm:defaults/fm:java_code/fm:location")) ==
                                  getElementText(pxd.selectSingleNode("/fm:application/fm:defaults/fm:java_code/fm:location"))));

            var jPackageSame = (cxd.selectSingleNode("/fm:application/fm:defaults/fm:java_code/fm:package") != null &&
                                pxd.selectSingleNode("/fm:application/fm:defaults/fm:java_code/fm:package") != null &&
                                (getElementText(cxd.selectSingleNode("/fm:application/fm:defaults/fm:java_code/fm:package")) ==
                                 getElementText(pxd.selectSingleNode("/fm:application/fm:defaults/fm:java_code/fm:package"))));

            var handlerPresent = (pxd.selectSingleNode("/fm:application/fm:handlers/fm:handler[@id = '" + this._id + "']") != null);


            //alert('locations the same: ' + jLocationSame + '\npackages the same: ' + jPackageSame + '\nHandler Present: ' + handlerPresent);

            if (!jLocationSame || !jPackageSame || !handlerPresent)
            {
                //alert('save required');
                requestContent =
                {
                    xml_project_info: document.result_form.xml_project_info.value,
                    action: 'process',
                    handler_id: this._id,
                    form_map_filename: document.result_form.form_map_filename.value,
                    xml_formMap: currentXML,
                    operation1: 'save_form_map',
                    operation2: 'load_java_code'
                }
            }
            else
            {
                //alert('save not needed');
                requestContent =
                {
                    xml_project_info: document.result_form.xml_project_info.value,
                    action: 'process',
                    handler_id: this._id,
                    form_map_filename: document.result_form.form_map_filename.value,
                    operation1: 'load_java_code'
                }
            }

            dojo.xhrPost(
                {
                    url: "FormMaker",
                    handleAs: "text",
                    preventCache: "true",
                    content: requestContent,
                    error: function(response, ioArgs)
                    {
                        alert('An error occurred while trying to load the document.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + response.message);
                        hideStatusMessage();
                        return response;
                    },
                    load: function(response, ioArgs)
                    {
                        hideStatusMessage();

                        var dojoResponse = getXMLDocument(response);
                        Sarissa.setXpathNamespaces(dojoResponse, "xmlns:fm='http://www.hyfinity.com/formmaker'");

                        var errorNode = dojoResponse.selectSingleNode("/fm:error");
                        if (errorNode != null)
                        {
                            alert('An error occurred while trying to load the document.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + getElementText(errorNode));
                        }
                        return response;
                    }
                });
        }
        else
        {
            alert('This option only applies to Java implementation Controllers.');
        }

    }

    function togglePageDefaults()
    {
        if (this._defaults == 'visible')
        {
            hideComponent(document.getElementById("create_new_controller_link"));
            hideComponent(document.getElementById("select_existing_controller_title"));
            hideComponent(document.getElementById("existing_controllers_select_box"));

            this._defaults = 'hidden';
            setTextContent(document.getElementById("show_handler_defaults"), 'Show Advanced Options');
        }
        else
        {
            showComponent(document.getElementById("create_new_controller_link"));
            showComponent(document.getElementById("select_existing_controller_title"));
            showComponent(document.getElementById("existing_controllers_select_box"));

            this._defaults = 'visible';
            setTextContent(document.getElementById("show_handler_defaults"), 'Hide Advanced Options');
        }
    }

    /**
     * Changes the implementation type of this handler to the specified value.
     * Then redisplays the context panel.
     * @param newValue The new implementiaon value to use - must be either 'xml' or 'java'
     */
    function implTypeChanged(newValue)
    {
        this._impl = newValue;
        this.displayContextPanel();
        this.setChanged();
    }


    //change detection/management functions
    function setChanged()
    {
        //alert('setting changed handler');
        this._hasChanges = true;
        this._container.setChanged();
    }

    function clearChanged()
    {
        this._hasChanges = false;
    }

}

//The number of pixels the mouse is allowed to be moved (in both x and y) to still be considered
//a click operation that shouldnt move the image
var allowedOffset = 1;

function differentMousePosition(firstPos, secondPos)
{
    if ((firstPos != null) && (secondPos != null) &&
        (Math.abs(secondPos.x - firstPos.x) < allowedOffset) &&
        (Math.abs(secondPos.y - firstPos.y) < allowedOffset))
    {
        return false;
    }
    else
    {
        return true;
    }
}
