/**
 * Page object
 * Encapsulates a Page in the formmaker application XML, provides utility methods to
 * output SVG and Formmaker XML.
 *
 * @param name Javascript reference name, needed when added callbacks in SVG output
 * @param container The parent object of this page
 * @param data Optional Data in XML or format to initialise the page, if not provided must set x, y, id separately
 */
function Page(name, container, data)
{
    this._name = name;
    this._container = container;
    this._mousedown = false;
    this._defaults = 'hidden';
    this._outgoingLinks = new Array();
    this._incomingLinks = new Array();
    this._forms = new Object();
    this._includedJavascriptFiles = new Array();
    this._includedCSSFiles = new Array();
    this._gennedSection = "";
    this._type = "";
    this._skin = "";
    this._targetType = "";
    this._sourceType = "";
    this._cssOverride = "";

    this._hasChanges = false;

    //Stores the XML string of the data sources listed against this page
    //this is just a quick change to ensure the data doesnt get lost
    //eventuially we will probabaly want to parse this information and allow editing of it.
    this._dataSourcesXML = "";

    //alert(this._startPage);

    if(typeof(data) != 'undefined')
    {
        this._id = data.getAttribute('id');
        this._x = Number(data.getAttribute('x'));
        this._y = Number(data.getAttribute('y'));
        this._type = data.getAttribute('type');

        //checks whether this is a normal page or subsection page.
        //will assign a type if one doesn't exist as of yet.
        this._type = data.getAttribute('type');
        if(this._type == null || this._type == "null" || this._type =="")
            this._type = "normal";

        //Deal with page attributes
        var skinNode = data.selectSingleNode("fm:skin");
        if(skinNode != null)
        {
            this._skin = getElementText(skinNode);
        }
        var cssNode = data.selectSingleNode("fm:css");
        if(cssNode != null)
        {
            this._includedCSSFiles[this._includedCSSFiles.length] = 'css/' + getElementText(cssNode);
        }
        var cssFiles = data.selectNodes("fm:css_files/fm:css_file");
        for(var i = 0; i < cssFiles.length; ++i)
        {
            this._includedCSSFiles[this._includedCSSFiles.length] = getElementText(cssFiles.item(i));
        }

        var targetTypeNode = data.selectSingleNode("fm:target_type");
        if(targetTypeNode != null)
        {
            this._targetType = getElementText(targetTypeNode);
        }
        var startPageNode = data.selectSingleNode("fm:start_page");
        if(startPageNode != null)
        {
          this._startPage = getElementText(startPageNode);
        }
        var cssOverrideNode = data.selectSingleNode("fm:css_override");
        if(cssOverrideNode != null)
        {
          this._cssOverride = getElementText(cssOverrideNode);
        }
        var accessibilityLevel = data.selectSingleNode("fm:accessibility_level");
        if(accessibilityLevel != null)
        {
          this._accessibilityLevel = getElementText(accessibilityLevel);
        }
        else
            this._accessibilityLevel = this._container._defaults._accessibilityLevel;

        var formRefs = data.selectNodes("fm:form_ref");
        for(var i = 0; i < formRefs.length; ++i)
        {
            var formRef = getElementText(formRefs.item(i));
            this._forms[formRef] = this._container._forms[formRef];
        }
        var includedJavascriptFiles = data.selectNodes("fm:script_files/fm:script_file");
        for(var i = 0; i < includedJavascriptFiles.length; ++i)
        {
            var includedJavascriptFile = getElementText(includedJavascriptFiles.item(i));
            this._includedJavascriptFiles[i] = includedJavascriptFile;
        }

        var dataSources = data.selectSingleNode("fm:data_sources");
        if(dataSources != null)
        {
            this._dataSourcesXML = xmlToString(dataSources);
        }

    }
    else
    {
        this._id = name;
        this._x = 0;
        this._y = 0;

        if(container._mode == "add_subsection")
            this._type = "subsection";
        else
            this._type = "normal";

        this._accessibilityLevel = this._container._defaults._accessibilityLevel;
        var pageCount = 0;
        for (index in this._container._pages)
            pageCount++;

        if (pageCount == 0)
            this._startPage = 'true' + '';
        else
            this._startPage = 'false' + '';
    }

    /*
    * Setup class methods
    */
    if (typeof(__PagePrototypeCalled) == 'undefined')
    {
        __PagePrototypeCalled = true;
        Page.prototype.draw = draw;
        Page.prototype.redraw = redraw;
        Page.prototype.getJavascriptIdentifier = getJavascriptIdentifier;
        Page.prototype.onmousedown = onmousedown;
        Page.prototype.onclick = onclick;
        Page.prototype.onmousemove = onmousemove;
        Page.prototype.onmouseup = onmouseup;
        Page.prototype.addHighlight = addHighlight;
        Page.prototype.removeHighlight = removeHighlight;
        Page.prototype.remove = remove;
        Page.prototype.rename = rename;
        Page.prototype.processNewName = processNewName;
        Page.prototype.addOutgoingLink = addOutgoingLink;
        Page.prototype.removeOutgoingLink = removeOutgoingLink;
        Page.prototype.addIncomingLink = addIncomingLink;
        Page.prototype.removeIncomingLink = removeIncomingLink;
        Page.prototype.togglePageDefaults = togglePageDefaults;
        Page.prototype.setStartPage = setStartPage;
        Page.prototype.getApplicationXML = getApplicationXML;
        Page.prototype.displayContextPanel = displayContextPanel;
        Page.prototype.toString = toString;
        Page.prototype.removeSkin = removeSkin;
        Page.prototype.setChanged = setChanged;
        Page.prototype.clearChanged = clearChanged;
    }

    /**
    * Create SVG for the Page and attach it to the containers SVG context
    */
    function draw()
    {
        SVGDocument = this._container.getSVGDocument();
        SVGContext = this._container.getSVGNodesContext();

        //Image
        /*var dummy = SVGDocument.getElementById('dummy_image');
        var image = dummy.cloneNode(true);
        image.setAttribute('id', "page_" + this._id);
        image.setAttribute('x', - Number(5));
        image.setAttribute('y', - Number(5));
        image.setAttribute('width', 10);
        image.setAttribute('height', 10);
        image.setAttribute('display', 'inline');*/

        var image = SVGDocument.createElementNS(SVG_NS, 'use');
        image.setAttribute('id', "page_" + this._id);
        image.setAttribute('x', - Number(5));
        image.setAttribute('y', - Number(5));
        image.setAttribute('width', 10);
        image.setAttribute('height', 10);

        if(this._type == "normal")
        {
            //image.setAttribute('xlink:href', '../images/view.png');
            //image.setAttributeNS(XLINK_NS, 'xlink:href', '../images/view.png');
            image.setAttributeNS(XLINK_NS, 'xlink:href', '#page');
        }
        else
        {
            //image.setAttribute('xlink:href', '../images/subsection.png');
            //image.setAttributeNS(XLINK_NS, 'xlink:href', '../images/subsection.png');
            image.setAttributeNS(XLINK_NS, 'xlink:href', '#partial_page');
        }

        //Text
        var text = SVGDocument.createElementNS(SVG_NS, 'text');
        var textData = SVGDocument.createTextNode(this._id);
        text.appendChild(textData);
        text.setAttribute('x', 0);
        text.setAttribute('y', 10);
        text.setAttribute('text-anchor', 'middle');
        text.setAttribute('pointer-events', 'none');
        text.setAttribute('style', 'font-size: 5px');

        //Group
        var group = SVGDocument.createElementNS(SVG_NS, 'g');
        group.setAttribute('id', this._id);
        group.setAttribute('transform', 'translate(' + this._x  + ', ' + this._y + ')');
        /*group.setAttribute('onmousedown', this.getJavascriptIdentifier() + '.onmousedown(evt)' );
        group.setAttribute('onclick', this.getJavascriptIdentifier() + '.onclick(evt)' );*/
        group.setAttribute('onmousemove', 'parent.' + this.getJavascriptIdentifier() + '.onmousemove(evt)' );
        group.setAttribute('onmouseup', "handleMouseUp('" + this.getJavascriptIdentifier() + "', evt)" );
        group.setAttribute('onmousedown', "handleMouseDown('" + this.getJavascriptIdentifier() + "', evt)" );
        group.setAttribute('onclick', "handleMouseClick('" + this.getJavascriptIdentifier() + "', evt)" );
        group.appendChild(image);
        group.appendChild(text);
        SVGContext.appendChild(group);

        //Maintain reference to group
        this._group = group;

        //Draw child links
        for(var i = 0; i < this._outgoingLinks.length; ++i)
        {
            if(this._outgoingLinks[i] != undefined)
            {
                this._outgoingLinks[i].redraw();
            }
        }

        //check if this page should still be highlighted
        if (this._container._selected == this)
        {
            this.addHighlight();
        }
    }

    /**
    * Adds a link to this page
    *
    * @param link The link to add
    */
    function addIncomingLink(link)
    {
        this._incomingLinks.push(link);
        this.setChanged();
    }

    /**
    * Removes a link to this page
    */
    function removeIncomingLink(link)
    {
        this._incomingLinks = removeElement(link, this._incomingLinks);
        for(var i = 0; i < this._incomingLinks.length; ++i)
        {
            if(this._incomingLinks[i] != undefined)
            {
                this._incomingLinks[i].redraw();
            }
        }
        this.setChanged();
    }

    /**
    * Adds a link to this page
    *
    * @param link The link to add
    */
    function addOutgoingLink(link)
    {
        this._outgoingLinks.push(link);
        //this.setChanged();
    }

    /**
    * Removes a link to this page
    */
    function removeOutgoingLink(link)
    {
        this._outgoingLinks = removeElement(link, this._outgoingLinks);
        for(var i = 0; i < this._outgoingLinks.length; ++i)
        {
            if(this._outgoingLinks[i] != undefined)
            {
                this._outgoingLinks[i].redraw();
            }
        }
        this.setChanged();
    }

    /**
    * Removes the skin file associated with this page
    */
    function removeSkin()
    {
        this._skin = "";
        this.displayContextPanel();
        this.setChanged();
    }

    /**
    * Delete old SVG for the page then redraw it
    */
    function redraw()
    {
        this._container.getSVGNodesContext().removeChild(this._group);
        this.draw();
    }

    /**
    * Mouse down event handler for this object
    */
    function onmousedown(event)
    {
        var mode = this._container._mode;
        switch(mode)
        {
            case 'select' :
                this._mousedown = true;
                this._container._moving = this;
                var screenCoords = new Object();
                screenCoords.x = event.clientX;
                screenCoords.y = event.clientY;
                this._mdposition = this._container._svgCoordinates.getUserSpaceCoordinates(screenCoords);
                break;

            /* The following has been commented to ensure the last target component does not act as the start of of the link by default. */
            /*case 'add_link':
                var linkID = this._id + '__temp__link';
                this._container._tempLink = new Link(linkID, null, this._container, this);
                this._container._tempLink.draw();
                this._container._tempLink._moving = true;
                break;*/
        }
    }

    /**
    * Mouse move event handler for this object
    */
    function onmousemove(event)
    {
        switch(this._container._mode)
        {
            case 'select':
                if(this._mousedown == true)
                {
                    this._container._svgCoordinates.reset();
                    this._moved = true;
                    var xdiff = this._container._svgCoordinates.getUserSpaceX(event.clientX);
                    var ydiff = this._container._svgCoordinates.getUserSpaceY(event.clientY);
                    var newPos = new Object();
                    newPos.x = xdiff;
                    newPos.y = ydiff;

                    if (differentMousePosition(this._mdposition, newPos) == true)
                    {
                        this._x = xdiff;
                        this._y = ydiff;
                        this._mdposition = null;
                        /*var transform = "translate(" + xdiff + ", " + ydiff + ")";
                        this._group.setAttribute('transform', transform);*/
                        this.redraw();

                        for(var i = 0; i < this._incomingLinks.length; ++i)
                        {
                            if(this._incomingLinks[i] != undefined)
                            {
                                this._incomingLinks[i].redraw();
                            }
                        }
                    }
                }
                break;

            case 'add_link':
                if(typeof(this._container._tempLink) != 'undefined')
                {
                    this._container._tempLink.onmousemove(event);
                }
                break;
        }
    }

    /**
    * Mouse up event handler
    */
    function onmouseup(event)
    {
        this._mousedown = false;
        this._container._moving = undefined;
        if(this._container._mode == 'select' && this._moved == true)
        {
            this._moved = false;
            var newx = this._container._svgCoordinates.getUserSpaceX(event.clientX);
            var newy = this._container._svgCoordinates.getUserSpaceY(event.clientY);
            var newPos = new Object();
            newPos.x = newx;
            newPos.y = newy;

            if (differentMousePosition(this._mdposition, newPos) == true)
            {
              this._x = newx;
              this._y = newy;
              this.redraw();
              this.setChanged();
            }
        }

        if(typeof(this._container._tempLink) != 'undefined')
        {
            var tempLink = this._container._tempLink;
            // do not allow links from handlers to partial pages
            if(this._type=='subsection' && tempLink._source.constructor == Handler && tempLink._source._type != 'thirdparty')
            {
                alert("Adding a link from a handler to a partial page is not allowed.");
                tempLink = null;
                this._container._tempLink.remove(this._container._tempLink);
            }
            // do not allow 3rd party service to partial page
            else if(this._type=='subsection' && tempLink._source.constructor == Handler && tempLink._source._type == 'thirdparty')
            {
                alert("Adding a link from a third party service to a partial page is not allowed.");
                tempLink = null;
                this._container._tempLink.remove(this._container._tempLink);
            }
            // do not allow 3rd party service to page
            else if(this._type=='normal' && tempLink._source.constructor == Handler && tempLink._source._type == 'thirdparty')
            {
                alert("Adding a link from a third party service to a page is not allowed.");
                tempLink = null;
                this._container._tempLink.remove(this._container._tempLink);
            }
            else if(this != tempLink._source)
            {
                if(tempLink._source.constructor == Page && this._type == 'subsection')
                {
                    tempLink._painterNode = this._container._defaults._painterNode;
                    var nodeArray = tempLink._painterNode.split('-');
                    tempLink._painterNodeID = nodeArray[nodeArray.length - 1];
                    tempLink._painterPattern = nodeArray[nodeArray.length - 2];
                    tempLink._submissionMethod = 'subsection';
                }
                tempLink._target = this;
                findUniqueLinkID(tempLink);
                this._container._links[tempLink._id] = tempLink;
                this.addIncomingLink(tempLink);
                tempLink._moving = false;
                this.redraw();
                tempLink._source.redraw();

                this._container.storeActionAddition(tempLink._action);
            }
            else
            {
                alert("Adding a link from a page to itself is not allowed.");
                tempLink = null;
                this._container._tempLink.remove(this._container._tempLink);
            }
            if (this._container._dblclick == false)
                this._container.setMode('select');
            this._container._tempLink = undefined;
        }
        //Bubble mouse up to ensure drag and drop works.
        document.onmouseup();
    }

    /**
    * Onclick event handler for this object
    */
    function onclick(event)
    {
        var mode = this._container._mode;
        switch(mode)
        {
            case 'select' :
                if (this._container._selected != this)
                {
                    selectedNode_nodeList = null;
                    this._container.setSelected(this);
                }
                break;
            case 'delete' :
                this.remove();
                if (this._container._dblclick == false)
                    this._container.setMode('select');
                break;
            case 'rename' :
                this.rename();
                if (this._container._dblclick == false)
                    this._container.setMode('select');
                break;
        }

        if (event.detail == 2)
            this.rename();
    }

    function showPageIDs(pages)
    {
        var pageString = "";
        for (page in pages)
            pageString += pages[page]._id + ' ';
    }

    /**
    * Gets a javascript string that identifies this object
    *
    * @returns A string containing javascript that identifies this object
    */
    function getJavascriptIdentifier()
    {
        return this._container.getPagesIdentifier() + "." + this._name;
    }

    /**
    * Highlights this page in the SVG
    */
    function addHighlight()
    {
        SVGDocument = this._container.getSVGDocument();
        var rect = SVGDocument.createElementNS(SVG_NS, 'rect');
        rect.setAttribute('stroke-width', 1);
        rect.setAttribute('opacity', 0.6);
        rect.setAttribute('fill', 'none');
        rect.setAttribute('stroke', 'red');
        rect.setAttribute('x', -5);
        rect.setAttribute('y', -5);
        rect.setAttribute('height', 10);
        rect.setAttribute('width', 10 );
        rect.setAttribute('id', 'highlight');

        this._group.appendChild(rect);
    }

    /**
    * Removes the highlight on this page in the SVG
    */
    function removeHighlight()
    {
        for(var i = 0; i < this._group.childNodes.length; ++i)
        {
            if(this._group.childNodes.item(i).getAttribute('id') == 'highlight')
            {
                this._group.removeChild(this._group.childNodes.item(i));
            }
        }
    }

    /**
    * Removes this page from the diagram
    */
    function remove()
    {
        showStatusMessage('Please wait...');

        this._container.getSVGNodesContext().removeChild(this._group);
        this._container.setSelected(undefined);
        this._container.setChanged();

        //make sure section is also removed
        for(index in this._forms)
        {
            var form_id = this._forms[index]._id
            deleteForm(this._id,this._forms[index]._id);
            break;
        }

        //remove handler if the page is a subsection
        if(this._type == 'subsection')
        {
            this._container._handlers[this._id].remove();
        }

        delete this._container._pages[this._name];

        //Need to make a copy of the links collections before deleting
        //because the remove method that gets called updates the collection
        //resulting in iteration problems

        //Remove all links associated with this page
        var tempIncomingLinks = cloneArray(this._incomingLinks);

        for(var i = 0; i < tempIncomingLinks.length; ++i)
        {
            if(tempIncomingLinks[i] != undefined)
            {
                tempIncomingLinks[i].remove(false);
            }
        }

        var tempOutgoingLinks = cloneArray(this._outgoingLinks);

        for(var i = 0; i < tempOutgoingLinks.length; ++i)
        {
            if(tempOutgoingLinks[i] != undefined)
            {
                tempOutgoingLinks[i].remove(false);
            }
        }
        dojo.xhrPost(
        {
            url: "FormMaker",
            handleAs: "text",
            preventCache: "true",
            content:
            {
                xml_project_info: document.result_form.xml_project_info.value,
                action: 'process',
                operation1: 'load_form_map',
                operation2: 'delete_page',
                page_name: this._name,
                form_map_filename: document.result_form.form_map_filename.value,
                binding_type: document.result_form.binding_type.value,
                form_id: form_id
            },
            pageDetails:
            {
                id: this._name
            },
            error: function(response, ioArgs)
            {
                alert('An error occurred while trying to load the information needed to show the appropriate document.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + response.message);
                hideStatusMessage();
                return response;
            },
            load: function(response, ioArgs)
            {
                hideStatusMessage();
            }
        });
    }

    /**
    * Enables inplace renaming of pages.
    */
    function rename()
    {
        var old_id = this._id
        var svgCoords = {x: this._x, y: this._y + 6}
        var screenCoords = this._container._svgCoordinates.getScreenCoordinates(svgCoords)
        /* this 'this' param represents the call back when a new name has been captured. */
        inplaceRenameCapture(screenCoords.x, screenCoords.y, old_id, this);
    }

    /**
    * This is the callback function from the inplaceRenameCapture function after a new name has been entered.
    * @param id The new name.
    */
    function processNewName(id)
    {
        var inputchek = false;
        var old_id = this._id;

        /* Use a flag to check for duplicate controller name on the app map when the page is a partial page. */
        bDuplicateControllerFound = false;
        //the form associated with this page will also be renamed to the new name, so we need to check uniqueness against the forms collection as well as the pages.
        inputchek = ((checkInvalidNames(id)) || (checkInvalidChars(id)) || (checkPropertyNameExists(id, this._container._pages)) || (checkPropertyNameExists(id, this._container._forms)));
        if (inputchek == false)
        {
            //check if it is a subsection. if it is handler needs renaming as well
            if(this._type == 'subsection')
            {
                /* Check for duplicate handlers. */
                if (!checkPropertyNameExists(id, this._container._handlers))
                {
                    this._container._handlers[old_id].processNewName(id);
                }
                else
                {
                    bDuplicateControllerFound = true;
                }
            }

            /* Only do this is a full page or a partial page with no duplicate handler name on the app map. */
            if (!bDuplicateControllerFound)
            {
                //call xstore to rename the underlying file
                //we should not update the FM application metadata untill we know this succeeded
                showStatusMessage('Please wait...');
                dojo.xhrPost(
                {
                    url: "FormMaker",
                    handleAs: "text",
                    preventCache: "true",
                    content:
                    {
                        xml_project_info: document.result_form.xml_project_info.value,
                        action: 'process',
                        operation1: 'rename_page',
                        binding_type: document.result_form.binding_type.value,
                        old_name: this._id,
                        new_name: id,
                        xml_formMap: this._container.getApplicationXML()
                    },
                    error: function(response, ioArgs)
                    {
                        alert('An error occurred while trying to rename the page.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + response.message);
                        hideStatusMessage();
                        return response;
                    },
                    load: function(response, ioArgs)
                    {
                        processPageRenameResult(response, ioArgs);
                        return response;
                    }
                });

                return true;
            }
        }
        if (inputchek || bDuplicateControllerFound)
        {
            /* Duplicate page name found or duplicate controller name found for a partial page - try again. Calling rename() here causes onblur to be constantly raised in the fmInplaceRenameUpdate. */
            //this.rename();
            return false;
        }
    }

    function togglePageDefaults()
    {
        if (this._defaults == 'visible')
        {
            var advancedOptionsRows = document.getElementById("page_details_table").getElementsByTagName("TR");
            for(var i=0;i<advancedOptionsRows.length;i++)
            {
                if(advancedOptionsRows[i].id != '')
                {
                    hideComponent(advancedOptionsRows[i]);
                }
            }

            this._defaults = 'hidden';
            setTextContent(document.getElementById("show_page_defaults"), 'Show Advanced Options');
        }
        else
        {
            var advancedOptionsRows = document.getElementById("page_details_table").getElementsByTagName("TR");
            for(var i=0;i<advancedOptionsRows.length;i++)
            {
                if(advancedOptionsRows[i].id != '')
                {
                    showComponent(advancedOptionsRows[i]);
                }
            }
            this._defaults = 'visible';
            setTextContent(document.getElementById("show_page_defaults"), 'Hide Advanced Options');
        }
    }

    function setStartPage()
    {
        this._startPage = document.result_form.default_start_page_flag.checked + '';
        if (this._startPage == 'true')
        {
            for (index in this._container._pages)
            {
                if ((this._container._pages[index]._startPage == 'true') & (this._container._pages[index]._id != this._id))
                    this._container._pages[index]._startPage = 'false';
            }
        }
        this.setChanged();
    }

    /**
    * Serializes the state of the diagram to application XML (as a string)
    */
    function getApplicationXML()
    {
        var xml = "<page xmlns='http://www.hyfinity.com/formmaker' ";
        xml += "id='" + this._id + "' ";
        xml += "x='" + this._x + "' ";
        xml += "y='" + this._y + "' ";
        xml += "type='" + this._type + "'>";
        if (this._skin != '')
            xml += "<skin>" + this._skin + "</skin>";
        /*if (this._css != '')
            xml += "<css>" + this._css + "</css>";*/
        xml += "<target_type>" + this._targetType + "</target_type>";

        if (this._startPage == '')
            this._startPage = 'false';
        if (this._cssOverride == '')
            this._cssOverride = 'false';
        if (typeof(this._accessibilityLevel) == 'undefined')
            this._accessibilityLevel = 'none';
        xml += "<start_page>" + this._startPage + "</start_page>";
        xml += "<css_override>" + this._cssOverride + "</css_override>";
        if ((this._accessibilityLevel != "") & (this._accessibilityLevel != this._container._defaults._accessibilityLevel))
            xml += "<accessibility_level>" + this._accessibilityLevel + "</accessibility_level>";

        for(form in this._forms)
        {
            xml += "<form_ref>" + form + "</form_ref>";
        }

        if (this._includedJavascriptFiles.length > 0)
        {
            xml+= "<script_files>";
            for(var i=0;i<this._includedJavascriptFiles.length;i++)
            {
                if(typeof(this._includedJavascriptFiles[i]) != "undefined")
                {
                    xml+="<script_file>" + escapeXMLCharacters(this._includedJavascriptFiles[i]) + "</script_file>";
                }
            }
            xml+= "</script_files>";
        }

        if (this._includedCSSFiles.length > 0)
        {
            xml+= "<css_files>";
            for(var i=0;i<this._includedCSSFiles.length;i++)
            {
                if(typeof(this._includedCSSFiles[i]) != "undefined")
                {
                    xml+="<css_file>" + escapeXMLCharacters(this._includedCSSFiles[i]) + "</css_file>";
                }
            }
            xml+= "</css_files>";
        }

        xml += this._dataSourcesXML;

        xml += "</page>";

        return xml;
    }

    /**
    * Displays the context panel for this page
    */
    function displayContextPanel()
    {
        this._container._pageTransform.addParameter('page_javascript_identifier', this.getJavascriptIdentifier());
        this._container._pageTransform.addParameter('page_id', this._id);
        this._container._pageTransform.addParameter('start_page', this._startPage);
        this._container._pageTransform.addParameter('genned_section', this._gennedSection);
        this._container._pageTransform.addParameter('css_override', this._cssOverride);
        this._container._pageTransform.addParameter('accessibility_level', this._accessibilityLevel);
        this._container._pageTransform.addParameter('page_type',this._container._selected._type);
        var xml = this._container.getApplicationXML();
        this._container._pageTransform.transform(getXMLDocument(xml));

        if (this._defaults == 'hidden')
            this._defaults = 'visible';
        else
            this._defaults = 'hidden';
        this.togglePageDefaults();

        for(form in this._forms)
        {
            showForm(form);
            break;
        }

        //detect whether the page is a subsection or not
        if(this._type == 'subsection')
        {
            for(handler in this._container._handlers)
            {
                if(this._name == handler)
                {
                    this._container._handlers[handler].displayContextPanel();
                }
            }
        }

        this._container.renderIncludedFilesList('PageJavascript');
        this._container.renderIncludedFilesList('PageCSS');
    }

    function toString()
    {
        return this._id;
    }


    //change detection/management functions
    function setChanged()
    {
        //alert('setting changed page');
        this._hasChanges = true;
        this._container.setChanged();
    }

    function clearChanged()
    {
        this._hasChanges = false;
    }


}


/**
 * Handles the response from the ajax call to xstore to rename a page.
 * If successful, this updates the FM appplication metadata with the new name.
 */
function processPageRenameResult(response,ioArgs)
{
    //check for a successful response
    var serverResponse = getXMLDocument(response);
    Sarissa.setXpathNamespaces(serverResponse, "xmlns:fm='http://www.hyfinity.com/formmaker'");
    //the request is considered a success if it succeeded, or if it failed because the source file did not exist.
    //this will happen if they are trying to rename a page that has not yet been generated.
    if (serverResponse.selectSingleNode("/fm:success"))
    {
        var originalName = ioArgs.args.content.old_name;
        var newName = ioArgs.args.content.new_name;
        var pageObj = navigationDiagram._pages[originalName];

        if (pageObj._incomingLinks.length != 0)
        {
            for(var i = 0; i < pageObj._incomingLinks.length; ++i)
            {
                if (pageObj._incomingLinks[i]._source.constructor == Handler)
                    pageObj._incomingLinks[i].removeExistingRule('page');
            }
        }

        delete pageObj._container._pages[originalName];

        pageObj._id = newName;

        pageObj._name = newName;
        pageObj._container._pages[pageObj._name] = pageObj;

        //rename the form associated with this page (assumes only one form)
        var originalFormName;
        for (f in pageObj._forms)
        {
            originalFormName = f;
            break;
        }
        pageObj._forms[originalFormName]._id = newName;
        pageObj._forms[newName] = pageObj._forms[originalFormName];
        delete pageObj._forms[originalFormName];
        delete pageObj._container._forms[originalFormName];
        pageObj._container._forms[newName] = pageObj._forms[newName];
        //update form document details
        var newDocName = serverResponse.selectSingleNode("/fm:success/fm:doc_name")
        if ((newDocName != null) && (getElementText(newDocName) != ''))
        {
            pageObj._container._forms[newName]._dataToForm._value = getElementText(newDocName);
        }

        if(pageObj._type != "subsection")
        {
            pageObj._container.setSelected(pageObj);
        }

        pageObj.redraw();
        pageObj.setChanged();

    }
    else
    {
        alert('We were unable to rename your page.\nPlease try again using a different name.\n\n');
        var originalName = ioArgs.args.content.old_name;
        var pageObj = navigationDiagram._pages[originalName];
        pageObj.rename();
    }

    hideStatusMessage();
}
